import Vue from 'vue';
import Vuetify from 'vuetify/lib/framework';

import { VueConstructor } from 'vue/types';

import VAD from './components/VApiDatatable/index.vue';
import VCD from './components/VCrudDatatable/index.vue';

import { REGISTRATION_PROPS, REGISTRATION_CRUD_PROPS } from './constants';

import { DatatablePluginOptions } from './components/types';

Vue.use(Vuetify);

const makeProps = (
  options: DatatablePluginOptions,
  availableProps: typeof REGISTRATION_PROPS & typeof REGISTRATION_CRUD_PROPS,
) =>
  Object.entries(options).reduce((acc, [key, value]) => {
    if (!availableProps[key as keyof DatatablePluginOptions]) return acc;
    return {
      ...acc,
      [key]: {
        ...availableProps[key as keyof DatatablePluginOptions],
        default: value,
      },
    };
  }, {});

export class VFetchDatatablePlguin {
  public install(Vue: VueConstructor, options: DatatablePluginOptions = {}) {
    const availableProps = {
      ...REGISTRATION_PROPS,
      ...REGISTRATION_CRUD_PROPS,
    };
    const extraProps = makeProps(options, availableProps);

    Vue.component('VCrudDatatable', {
      extends: VCD,
      props: extraProps,
    });

    Vue.component('VApiDatatable', {
      extends: VAD,
      props: extraProps,
    });
  }
}

export const VFetchDatatable = VAD;

export default new VFetchDatatablePlguin();
